#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/8/16 20:51
# @Author  : jiaguoqing 
# @Email   : jiaguoqing12138@gmail.com
# @File    : distances.py


def elu_distance(vector1, vector2):
    """
    计算两个向量间的欧氏距离。
    """
    x = sum([pow(v1 - v2, 2) for v1, v2 in zip(vector1, vector2)])
    return x



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/8/16 20:55
# @Author  : jiaguoqing 
# @Email   : jiaguoqing12138@gmail.com
# @File    : tf-idf.py


class tfidf(object):
    """
    将文档转换为 TF-IDF 特征表示
    """
    def __init__(self, text):
        self.text = text



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/8/17 16:26
# @Author  : jiaguoqing 
# @Email   : jiaguoqing12138@gmail.com
# @File    : __init__.py

from wuyi.tokenizers.BasicTokenizer import BasicTokenizer
from wuyi.metric.ROUGE import ROUGE
from wuyi.metric.BLEU import BLEU



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/8/30 19:30
# @Author  : jiaguoqing 
# @Email   : jiaguoqing12138@gmail.com
# @File    : BLEU.py

import math
import sys
from collections import defaultdict

from wuyi.tokenizers.BasicTokenizer import BasicTokenizer


def _ngram(sent, n_size):
    ngram_list = []
    for left in range(len(sent) - n_size):
        ngram_list.append(sent[left:left + n_size + 1])
    return ngram_list


def get_ngram(sent, n_size, label=None):
    ngram_list = _ngram(sent, n_size)
    if label is not None:
        ngram_list = [ngram + '_' + label for ngram in ngram_list]
    return ngram_list


def get_match_size(hyp_ngram, ref_ngram):
    # ref
    ref_set = defaultdict(int)
    # for r_n in ref_ngram:
    #     tmp_ref_set = defaultdict(int)
    #     for n in r_n:
    #         tmp_ref_set[tuple(n)] += 1
    #     for ngram, count in tmp_ref_set.items():
    #         ref_set[tuple(ngram)] = max(ref_set[tuple(ngram)], count)
    for ngram in ref_ngram:
        ref_set[tuple(ngram)] += 1

    # hyp
    hyp_set = defaultdict(int)
    for ngram in hyp_ngram:
        hyp_set[tuple(ngram)] += 1

    # match
    match_size = 0
    for ngram, count in hyp_set.items():
        match_size += min(count, ref_set.get(tuple(ngram), 0))  # 匹配次数
    hyp_size = len(hyp_ngram)

    return match_size, hyp_size


class BLEU:
    def __init__(self, n_size=4, weights=None):
        if not weights:
            weights = [1 / n_size for _ in range(n_size)]
        assert len(weights) == n_size, (
                "Number of weights and n-gram should be the same, got Number of weights: '%d' and n-gram: '%d'"
                % (len(weights), n_size))
        self.weights = weights
        self.n_size = n_size
        # tokenizer
        self.tokenizer = BasicTokenizer()

    def get_scores(self, hyps, refs, ignore_empty=False):
        if isinstance(hyps, str):  # 当输入为单条数据，转为List
            hyps, refs = [hyps], [refs]

        if ignore_empty:  # 去掉空值
            hyps_and_refs = [(hyp, ref) for hyp, ref in zip(hyps, refs)
                             if len(hyp) > 0 and len(ref) > 0]
            hyps, refs = zip(*hyps_and_refs)

        assert isinstance(hyps, type(refs)), "type of `hyps` is {}, type of `refs` is {}".format(type(hyps), type(refs))
        assert len(hyps) == len(refs), "length of `hyps` is {}, length of `ref` is {}".format(len(hyps), len(refs))

        return self._get_scores(hyps, refs)

    def _get_scores(self, hyps, refs):
        # add inst
        match_ngram = {}
        candi_ngram = {}
        bp_c = 0
        bp_r = 0
        for hyp, ref in zip(hyps, refs):
            hyp = self.tokenizer.tokenize(hyp)
            ref = self.tokenizer.tokenize(ref)
            for n_size in range(self.n_size):
                hyp_ngram = get_ngram(hyp, n_size)
                ref_ngram = get_ngram(ref, n_size)
                if n_size not in match_ngram:
                    match_ngram[n_size] = 0
                    candi_ngram[n_size] = 0
                match_size, hyp_size = get_match_size(hyp_ngram, ref_ngram)
                match_ngram[n_size] += match_size
                candi_ngram[n_size] += hyp_size
            bp_c += len(hyp)
            bp_r += len(ref)

        # Calculate the final bleu metric.
        prob_list = []
        for n_size in range(self.n_size):
            try:
                if candi_ngram[n_size] == 0:
                    _score = 0.0
                else:
                    _score = match_ngram[n_size] / float(candi_ngram[n_size])
            except:
                _score = 0
            if _score == 0:
                _score = sys.float_info.min
            prob_list.append(_score)

        logs = math.fsum(w_i * math.log(p_i) for w_i, p_i in zip(self.weights, prob_list))
        bp = math.exp(min(1 - bp_r / float(bp_c), 0))
        bleu = bp * math.exp(logs)
        return bleu



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/8/30 19:31
# @Author  : jiaguoqing 
# @Email   : jiaguoqing12138@gmail.com
# @File    : ROUGE.py

""" 计算ROUGE分数 """

from wuyi.metric import rouge_score
from wuyi.tokenizers.BasicTokenizer import BasicTokenizer


class ROUGE:
    DEFAULT_METRICS = ["rouge-1", "rouge-2", "rouge-l"]
    METRICS_FN = {
        "rouge-1": lambda hyp, ref, **k: rouge_score.rouge_n(hyp, ref, 1, **k),
        "rouge-2": lambda hyp, ref, **k: rouge_score.rouge_n(hyp, ref, 2, **k),
        "rouge-l": lambda hyp, ref, **k: rouge_score.rouge_l_summary_level(hyp, ref, **k),
    }
    DEFAULT_STATS = ['f', 'p', 'r']

    def __init__(self, metrics=None, stats=None):
        # metrics
        if metrics is not None:
            if isinstance(metrics, list):
                self.metrics = [m.lower() for m in metrics]
            else:
                self.metrics = [metrics.lower()]

            for m in self.metrics:
                if m not in ROUGE.DEFAULT_METRICS:
                    raise ValueError("Unknown metric '%s'" % m)
        else:
            self.metrics = ROUGE.DEFAULT_METRICS

        # states
        if stats is not None:
            self.stats = [s.lower() for s in stats]
            for s in self.stats:
                if s not in ROUGE.DEFAULT_STATS:
                    raise ValueError("Unknown stat '%s'" % s)
        else:
            self.stats = ROUGE.DEFAULT_STATS

        # tokenizer
        self.tokenizer = BasicTokenizer()

    def get_scores(self, hyps, refs, avg=False, ignore_empty=False):
        if isinstance(hyps, str):  # 当输入为单条数据，转为List
            hyps, refs = [hyps], [refs]

        if ignore_empty:  # 去掉空值
            hyps_and_refs = [(hyp, ref) for hyp, ref in zip(hyps, refs)
                             if len(hyp) > 0 and len(ref) > 0]
            hyps, refs = zip(*hyps_and_refs)

        assert isinstance(hyps, type(refs)), "type of `hyps` is {}, type of `refs` is {}".format(type(hyps), type(refs))
        assert len(hyps) == len(refs), "length of `hyps` is {}, length of `ref` is {}".format(len(hyps), len(refs))

        if not avg:
            return self._get_scores(hyps, refs)
        return self._get_avg_scores(hyps, refs)

    def _get_scores(self, hyps, refs):
        scores = []
        for hyp, ref in zip(hyps, refs):
            sen_score = {}

            hyp = ' '.join(self.tokenizer.tokenize(hyp))
            ref = ' '.join(self.tokenizer.tokenize(ref))
            hyp = [' '.join(_.split()) for _ in hyp.split('。') if len(_) > 0]
            ref = [' '.join(_.split()) for _ in ref.split('。') if len(_) > 0]

            for m in self.metrics:
                fn = ROUGE.METRICS_FN[m]
                sc = fn(hyp, ref)
                sen_score[m] = {s: sc[s] for s in self.stats}
            scores.append(sen_score)
        return scores

    def _get_avg_scores(self, hyps, refs):
        scores = {m: {s: 0 for s in self.stats} for m in self.metrics}

        count = 0
        for hyp, ref in zip(hyps, refs):
            hyp = ' '.join(self.tokenizer.tokenize(hyp))
            ref = ' '.join(self.tokenizer.tokenize(ref))
            hyp = [' '.join(_.split()) for _ in hyp.split('。') if len(_) > 0]
            ref = [' '.join(_.split()) for _ in ref.split('。') if len(_) > 0]

            for m in self.metrics:
                fn = ROUGE.METRICS_FN[m]
                sc = fn(hyp, ref)
                scores[m] = {s: scores[m][s] + sc[s] for s in self.stats}

            count += 1

        avg_scores = {m: {s: scores[m][s] / count for s in self.stats} for m in self.metrics}
        return avg_scores



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/8/31 9:57
# @Author  : jiaguoqing 
# @Email   : jiaguoqing12138@gmail.com
# @File    : BasicTokenizer.py

import unicodedata


def _is_whitespace(char):
    """Checks whether `char` is a whitespace character."""
    # \t, \n, and \r are technically control characters but we treat them
    # as whitespace since they are generally considered as such.
    if char == " " or char == "\t" or char == "\n" or char == "\r":
        return True
    cat = unicodedata.category(char)
    if cat == "Zs":
        return True
    return False


def _is_control(char):
    """Checks whether `char` is a control character."""
    # These are technically control characters but we count them as whitespace
    # characters.
    if char == "\t" or char == "\n" or char == "\r":
        return False
    cat = unicodedata.category(char)
    if cat.startswith("C"):
        return True
    return False


def _is_punctuation(char):
    """Checks whether `char` is a punctuation character."""
    cp = ord(char)
    # We treat all non-letter/number ASCII as punctuation.
    # Characters such as "^", "$", and "`" are not in the Unicode
    # Punctuation class but we treat them as punctuation anyways, for
    # consistency.
    if (33 <= cp <= 47) or (58 <= cp <= 64) or (91 <= cp <= 96) or (123 <= cp <= 126):
        return True
    cat = unicodedata.category(char)
    if cat.startswith("P"):
        return True
    return False


def _is_end_of_word(text):
    """Checks whether the last character in text is one of a punctuation, control or whitespace character."""
    last_char = text[-1]
    return bool(_is_control(last_char) | _is_punctuation(last_char) | _is_whitespace(last_char))


def _is_start_of_word(text):
    """Checks whether the first character in text is one of a punctuation, control or whitespace character."""
    first_char = text[0]
    return bool(_is_control(first_char) | _is_punctuation(first_char) | _is_whitespace(first_char))


def whitespace_tokenize(text):
    """Runs basic whitespace cleaning and splitting on a piece of text."""
    text = text.strip()
    if not text:
        return []
    tokens = text.split()
    return tokens


class BasicTokenizer(object):
    """ BasicTokenizer """
    def __init__(self, do_lower_case=True, never_split=None, tokenize_chinese_chars=True, strip_accents=None):
        if never_split is None:
            never_split = []
        self.do_lower_case = do_lower_case
        self.never_split = set(never_split)
        self.tokenize_chinese_chars = tokenize_chinese_chars
        self.strip_accents = strip_accents

    def tokenize(self, text, never_split=None):
        """
        Basic Tokenization of a piece of text. Split on "white spaces" only, for sub-word tokenization, see
        WordPieceTokenizer.

        Args:
            **never_split**: (`optional`) list of str
                Kept for backward compatibility purposes. Now implemented directly at the base class level (see
                :func:`PreTrainedTokenizer.tokenize`) List of token not to split.
        """
        # union() returns a new set by concatenating the two sets.
        never_split = self.never_split.union(set(never_split)) if never_split else self.never_split
        text = self._clean_text(text)

        # This was added on November 1st, 2018 for the multilingual and Chinese
        # models. This is also applied to the English models now, but it doesn't
        # matter since the English models were not trained on any Chinese data
        # and generally don't have any Chinese data in them (there are Chinese
        # characters in the vocabulary because Wikipedia does have some Chinese
        # words in the English Wikipedia.).
        if self.tokenize_chinese_chars:
            text = self._tokenize_chinese_chars(text)
        orig_tokens = whitespace_tokenize(text)
        split_tokens = []
        for token in orig_tokens:
            if token not in never_split:
                if self.do_lower_case:
                    token = token.lower()
                    if self.strip_accents is not False:
                        token = self._run_strip_accents(token)
                elif self.strip_accents:
                    token = self._run_strip_accents(token)
            split_tokens.extend(self._run_split_on_punc(token, never_split))

        ouput_tokens = whitespace_tokenize(' '.join(split_tokens))
        return ouput_tokens

    def _run_strip_accents(self, text):
        """Strips accents from a piece of text."""
        text = unicodedata.normalize("NFD", text)
        output = []
        for char in text:
            cat = unicodedata.category(char)
            if cat == "Mn":
                continue
            output.append(char)
        return "".join(output)

    def _run_split_on_punc(self, text, never_split=None):
        """Splits punctuation on a piece of text."""
        if never_split is not None and text in never_split:
            return [text]
        chars = list(text)
        i = 0
        start_new_word = True
        output = []
        while i < len(chars):
            char = chars[i]
            if _is_punctuation(char):
                output.append([char])
                start_new_word = True
            else:
                if start_new_word:
                    output.append([])
                start_new_word = False
                output[-1].append(char)
            i += 1

        return ["".join(x) for x in output]

    def _tokenize_chinese_chars(self, text):
        """Adds whitespace around any CJK character."""
        output = []
        for char in text:
            cp = ord(char)
            if self._is_chinese_char(cp):
                output.append(" ")
                output.append(char)
                output.append(" ")
            else:
                output.append(char)
        return "".join(output)

    def _is_chinese_char(self, cp):
        """Checks whether CP is the codepoint of a CJK character."""
        # This defines a "chinese character" as anything in the CJK Unicode block:
        #   https://en.wikipedia.org/wiki/CJK_Unified_Ideographs_(Unicode_block)
        #
        # Note that the CJK Unicode block is NOT all Japanese and Korean characters,
        # despite its name. The modern Korean Hangul alphabet is a different block,
        # as is Japanese Hiragana and Katakana. Those alphabets are used to write
        # space-separated words, so they are not treated specially and handled
        # like the all of the other languages.
        if (
                (0x4E00 <= cp <= 0x9FFF)
                or (0x3400 <= cp <= 0x4DBF)  #
                or (0x20000 <= cp <= 0x2A6DF)  #
                or (0x2A700 <= cp <= 0x2B73F)  #
                or (0x2B740 <= cp <= 0x2B81F)  #
                or (0x2B820 <= cp <= 0x2CEAF)  #
                or (0xF900 <= cp <= 0xFAFF)
                or (0x2F800 <= cp <= 0x2FA1F)  #
        ):  #
            return True

        return False

    def _clean_text(self, text):
        """Performs invalid character removal and whitespace cleanup on text."""
        output = []
        for char in text:
            cp = ord(char)
            if cp == 0 or cp == 0xFFFD or _is_control(char):
                continue
            if _is_whitespace(char):
                output.append(" ")
            else:
                output.append(char)
        return "".join(output)


class WordpieceTokenizer(object):
    """Runs WordPiece tokenization."""

    def __init__(self, vocab, unk_token, max_input_chars_per_word=100):
        self.vocab = vocab
        self.unk_token = unk_token
        self.max_input_chars_per_word = max_input_chars_per_word

    def tokenize(self, text):
        """
        Tokenizes a piece of text into its word pieces. This uses a greedy longest-match-first algorithm to perform
        tokenization using the given vocabulary.

        For example, :obj:`input = "unaffable"` wil return as output :obj:`["un", "##aff", "##able"]`.

        Args:
          text: A single token or whitespace separated tokens. This should have
            already been passed through `BasicTokenizer`.

        Returns:
          A list of wordpiece tokens.
        """

        output_tokens = []
        for token in whitespace_tokenize(text):
            chars = list(token)
            if len(chars) > self.max_input_chars_per_word:
                output_tokens.append(self.unk_token)
                continue

            is_bad = False
            start = 0
            sub_tokens = []
            while start < len(chars):
                end = len(chars)
                cur_substr = None
                while start < end:
                    substr = "".join(chars[start:end])
                    if start > 0:
                        substr = "##" + substr
                    if substr in self.vocab:
                        cur_substr = substr
                        break
                    end -= 1
                if cur_substr is None:
                    is_bad = True
                    break
                sub_tokens.append(cur_substr)
                start = end

            if is_bad:
                output_tokens.append(self.unk_token)
            else:
                output_tokens.extend(sub_tokens)
        return output_tokens



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="WuYi",
    version="0.0.6",
    author="Jia Guoqing",
    author_email="jiaguoqing12138@gmail.com",
    description="A small NLP tool package.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Moriarty12138/WuYi",
    project_urls={
        "Bug Tracker": "https://github.com/Moriarty12138/WuYi/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=['wuyi'],
    package_dir={'wuyi': 'wuyi'},
    package_data={'wuyi': ['*.*', 'clustering/*', 'core/*', 'metric/*', 'tokenizers/*']},
    python_requires=">=3.6",
)